#include "EGL/egl.h"
#include "EGL/eglext.h"
#include <cstdlib>
#include <iostream>

using namespace std;

//
//  Abfrage der Eigenschaften von EGL
//  Munshi, Aaftab; Ginsburg, Dan; Shreiner, Dave:
//  OpenGL ES 2.0 Programming Guide
//  Addison-Wesley, July 2008
//  ISBN-13:  978-0-321-50279-7
//  ISBN-10:      0-321-50279-5
//   Seite 38-39
//

const char* decodeAPISupportStrings[] = {
    "No API support(?)",
    "OpenGL ES",
    "OpenVG",
    "OpenGL ES, OpenVG",
    "OpenGL ES 2.0",
    "OpenGL ES, OpenGL ES 2.0",
    "OpenGL ES 2.0, OpenVG",
    "OpenGL ES, OpenGL ES 2.0, OpenVG"
};
/* Decode EGL_SURFACE_TYPE */
const char* decodeSurfaceStrings[] = {
    "None!",
    "PBuffer",
    "Pixmap",
    "PBuffer+Pixmap",
    "Window",
    "Window+PBuffer",
    "Window+Pixmap",
    "Window+Pixmap+PBuffer"
};

void decodeConfiguration(int i, EGLDisplay display, EGLConfig config);

int main(int  argc, char *argv[]) {
    EGLint majorVersion;
    EGLint minorVersion;
    EGLDisplay display;
    EGLBoolean result;
    EGLint numConfigs;
    
    cout << "Examination of EGL" << endl;
    
    display = eglGetDisplay(EGL_DEFAULT_DISPLAY);
    if (display == EGL_NO_DISPLAY) {
        cout << "unable to open connection to EGL display" << endl;
        exit (1);
    }
    if (!eglInitialize(display, &majorVersion, &minorVersion)) {
        cout << "Unable to initialize EGL" << endl;
        exit (1);
    } else {
        cout << "EGL version " << majorVersion << ":" << minorVersion << endl;
    }
    
    //  erste Anfrage: Anzahl der Konfigurationen erfragen
    result = eglGetConfigs (display, NULL, 20, &numConfigs);
    if (result != EGL_TRUE) {
        cout << "failure in eglGetConfigs" << endl;
        exit(1);
    }
    cout << "number of available configurations: " << numConfigs << endl;
    
    //  zweite Anfrage: alle Konfigurationen erfragen
    EGLConfig configs [numConfigs];
    EGLint nc;
    result = eglGetConfigs (display, &configs[0], numConfigs, &nc);
    
    for (int i = 0; i < nc; i++) {
        decodeConfiguration(i, display, configs[i]);
    }
    cout << "--- end of report ---" << endl;
    return 0;
}

void decodeRenderableType(EGLint intVal) {
	    /* Ignores OpenGL (desktop) support (bit 3 set) */
    cout << "      " << decodeAPISupportStrings[intVal & 7] <<endl;
}


void decodeSurfaceType(EGLint intVal)
{
    /* TODO consider more than just bits 0-2 */
    cout << "       " << decodeSurfaceStrings[intVal & 7] << endl;
}

const char* decodeColorBuffer(EGLint value)
{
    switch(value)
    {
        case EGL_RGB_BUFFER:
            return "RGB colour buffer";
            break;
        case EGL_LUMINANCE_BUFFER:
            return "Luminance buffer";
            break;
        default:
            return "Unknown EGL_COLOR_BUFFER_TYPE";
            break;
    }
}

void decodeConfiguration(int i, EGLDisplay display, EGLConfig config) {
	
    EGLBoolean result;
    EGLint intVal;
    
    result = eglGetConfigAttrib(display, config, EGL_BUFFER_SIZE, &intVal);
    cout << i << " :  " << "EGL_BUFFER_SIZE  " << intVal << endl;
        
    result = eglGetConfigAttrib(display, config, EGL_RED_SIZE, &intVal);
    cout << i << " :  " << "EGL_RED_SIZE     " << intVal << endl;
        
    result = eglGetConfigAttrib(display, config, EGL_GREEN_SIZE, &intVal);
    cout << i << " :  " << "EGL_GREEN_SIZE   " << intVal << endl;
        
    result = eglGetConfigAttrib(display, config, EGL_BLUE_SIZE, &intVal);
    cout << i << " :  " << "EGL_BLUE_SIZE    " << intVal << endl;
        
    result = eglGetConfigAttrib(display, config, EGL_LUMINANCE_SIZE, &intVal);
    cout << i << " :  " << "EGL_LUMINANCE_SIZE    " << intVal << endl;
        
    result = eglGetConfigAttrib(display, config, EGL_ALPHA_SIZE, &intVal);
    cout << i << " :  " << "EGL_ALPHA_SIZE    " << intVal << endl;
        
    result = eglGetConfigAttrib(display, config, EGL_ALPHA_MASK_SIZE, &intVal);
    cout << i << " :  " << "EGL_ALPHA_MASK_SIZE    " << intVal << endl;
        
    result = eglGetConfigAttrib(display, config, EGL_DEPTH_SIZE, &intVal);
    cout << i << " :  " << "EGL_DEPTH_SIZE    " << intVal << endl;
        
    result = eglGetConfigAttrib(display, config, EGL_SAMPLE_BUFFERS, &intVal);
    cout << i << " :  " << "EGL_SAMPLE_BUFFERS    " << intVal << endl;
        
    result = eglGetConfigAttrib(display, config, EGL_SAMPLES, &intVal);
    cout << i << " :  " << "EGL_SAMPLES    " << intVal << endl;
        
    result = eglGetConfigAttrib(display, config, EGL_STENCIL_SIZE, &intVal);
    cout << i << " :  " << "EGL_STENCIL_SIZE    " << intVal << endl;
        
    result = eglGetConfigAttrib(display, config, EGL_LEVEL, &intVal);
    cout << i << " :  " << "EGL_LEVEL   " << intVal << endl;
        
    result = eglGetConfigAttrib(display, config, EGL_MAX_PBUFFER_WIDTH, &intVal);
    cout << i << " :  " << "EGL_MAX_PBUFFER_WIDTH    " << intVal << endl;
        
    result = eglGetConfigAttrib(display, config, EGL_MAX_PBUFFER_HEIGHT, &intVal);
    cout << i << " :  " << "EGL_MAX_PBUFFER_HEIGHT    " << intVal << endl;
        
    result = eglGetConfigAttrib(display, config, EGL_MAX_PBUFFER_PIXELS, &intVal);
    cout << i << " :  " << "EGL_MAX_PBUFFER_PIXELS    " << intVal << endl;
        
    result = eglGetConfigAttrib(display, config, EGL_RENDERABLE_TYPE, &intVal);
    cout << i << " :  " << "EGL_RENDERABLE_TYPE    " << intVal << endl;
    decodeRenderableType(intVal);
        
    result = eglGetConfigAttrib(display, config, EGL_SURFACE_TYPE, &intVal);
    cout << i << " :  " << "EGL_SURFACE_TYPE    " << intVal << endl;
    decodeSurfaceType(intVal);
        
    result = eglGetConfigAttrib(display, config, EGL_COLOR_BUFFER_TYPE, &intVal);
    cout << i << " :  " << "EGL_COLOR_BUFFER_TYPE    " << decodeColorBuffer(intVal) << endl;
   
         
    cout << endl;    
}
